def unique_formula(*groups,group_labels=[]):
    
    """ 
	Docstring for function pyKrev.unique_formula
    
	====================
	This function compares n lists of molecular formula and outputs a dictionary containing the unique formula in each list.
    
	Use
	----
	unique_formula(list_1,..,list_n)
    
	Returns a dictionary in which each key corresponds to an input list (labelled "group 1...n") 
	and the corresponding value is a list containing the unique formula in that group.  
    
	Parameters
	----------
	*groups: n lists of molecular formula. Each item in the list should be a formula string. 
	 group_labels = list of group label strings of len(groups)
    
    """
    default_names = ['group 1','group 2','group 3','group 4','group 5','group 6','group 7','group 8','group 9']

    if not group_labels:
        group_labels = default_names[0:len(groups)]

    molecular_formula = dict()
    unique_molecular_formula = dict()
    
    for i,g in zip(group_labels,groups): 
        molecular_formula[i] = g 
    
    
    #now compare elements of the groups 
    for i in group_labels: #zip so that we don't extend pass length of gro
        #create a shallow copy of the molecular formula dictionary 
        temp_molecular_formula = dict.copy(molecular_formula)
        #create a set out of the unique formulas in the current group 
        current_group = set(molecular_formula[i])
        #remove the current group from the shallow copy
        del temp_molecular_formula[i]
        #create a set out of the unique formulas in the shallow copy
        total = []
        temp_values = list(temp_molecular_formula.values())
        for f in temp_values:
            total += f #merge the sub lists
        other_groups = set(total)
        #compare current_group to the unique formulas in all other groups
        unique_formula  = []
        for formula in current_group:
            if formula in other_groups:
                pass
            else:
                unique_formula.append(formula)
        #add it to the unique_molecular_formula dict 
        unique_molecular_formula[i] = unique_formula
    
    
    return unique_molecular_formula


from ..formula import kendrick_mass_defect
from matplotlib import pyplot as plt
def kendrick_mass_defect_plot(formula_list,mz_list, base = 'CH2', rounding = 'even', **kwargs):
    """ 
	Docstring for function PyKrev.kendrick_mass_defect_plot.py
	====================
	This function takes a list of molecular formula strings and a list or numpy array of mz values and creates a kendrick mass defect plot.
	Use
	----
	kendrick_mass_defect_plot(Y,X)
    
	Returns the figure and axes handles and a tuple containing two numpy arrays, the first contains the kendrick mass and the second the kendrick mass defect. 
    
	Parameters
	----------
	Y: A list of molecular formula. 
    X: A list or numpy array of mz_values of len(Y). 
	Base: Atom group used to define the Kendrick mass.
    **kwargs: key word arguments for pyplot.scatter(). 
    Info
	----------
    Calculation taken from Hughey et al (2001) 
    "Kendrick Mass Defect Spectrum: A Compact Visual Analysis for Ultrahigh-Resolution Broadband Mass Spectra"
    Note: Rounding calclations may lead to artefacts in complex datasets with many peaks. 
    We recommend experimenting with both even and integer rounding methods when making kmd plots.
    """
    
    kendrickMass, kendrickMassDefect = kendrick_mass_defect(formula_list,mz_list,base=base,rounding=rounding)
    plt.scatter(kendrickMass,kendrickMassDefect, **kwargs)
    plt.xlabel('Kendrick Mass')
    plt.ylabel('Kendrick Mass Defect')
    fig = plt.gcf()
    ax = plt.gca()
    return fig, ax, (kendrickMass, kendrickMassDefect)
    


from .aromaticity_index import aromaticity_index 
from .double_bond_equivalent import double_bond_equivalent 
from .element_counts import element_counts
from .element_ratios import element_ratios
from .missing_formula import missing_formula 
from .nominal_oxidation_state import nominal_oxidation_state
from .standardize_formula import standardize_formula 
from .unique_formula import unique_formula 
from .find_intersections import find_intersections
from .read_formularity import read_formularity
from .read_batch_formularity import read_batch_formularity
from .calculate_mass import calculate_mass
from .kendrick_mass_defect import kendrick_mass_defect
from .filter_spectral_interference import filter_spectral_interference


import pandas as pd
import numpy as np 

def read_formularity(report_name,pi_col = []):
    """ 
	Docstring for function PyKrev.read_formularity
	====================
	This function reads the report csv file produced by formularity software and saves the formula and associated parameters as variables in python. 
	The function filters out any formula that do not have C and H atoms.
    
	Use
	----
	read_formularity(report_name)
    
	Returns a list of formula, a numpy array of peak intensities, a numpy array of mz ratios, a numpy of mass errors and a list of compound class assignments. 
    
	Parameters
	----------
	report_name: name of csv file that the formularity report to be read is saved as. 
	pi_col: name of the column in that file that peak intensities are found in. If not given the last column is used. 
    """
        
    report = pd.read_csv(report_name)
    
    #sum C and C13 atoms
    C = report['C'] + report['C13']
    H = report['H']
    O = report['O']
    N = report['N'] 
    S = report['S']
    P = report['P']

 
    if not pi_col: 
        I = report.iloc[:,-1] #take the final column of the report file to contain peak intensities. Not sure how stable this is.
    else:
        I = report[pi_col] #user supplied a column name for the peak intensities.
    
    MZ = report['Mass']
    G = report['Class']
    ME = report['Error_ppm']

    molecular_formula = []
    mass_charge = np.array([])
    mass_error = np.array([])
    peak_intensities = np.array([])
    compound_class = []
    
    for n in range(0,len(C)):
        if C[n] == 0 and H[n] == 0: #If there isn't a count for C and H don't include the formula
            pass
        else:
            temp_formula = 'C'+str(C[n])+'H'+str(H[n]) #add C and H then ... 
            if N[n] > 0:
                temp_formula = temp_formula + 'N' + str(N[n])
            if O[n] > 0:
                temp_formula = temp_formula + 'O' + str(O[n])
            if P[n] > 0:
                temp_formula = temp_formula + 'P' + str(P[n])
            if S[n] > 0:
                temp_formula = temp_formula + 'S' + str(S[n])
            
            molecular_formula.append(temp_formula)
            peak_intensities = np.append(peak_intensities,I[n])
            mass_charge =  np.append(mass_charge,MZ[n])
            mass_error = np.append(mass_error,ME[n])
            compound_class.append(G[n])
       
    return molecular_formula, peak_intensities, mass_charge, mass_error, compound_class #return as a tuple, that can be unpacked if necessary


from ..diversity import normalise_intensity
from ..formula import calculate_mass
from matplotlib import pyplot as plt
import pandas as pd
import numpy as np

def mass_spectrum(formula_list, 
                  peak_intensities,
                  mz_list = [],
                  normalise = True,
                  method = 'monoisotopic',
                  **kwargs):
    """ 
	Docstring for function PyKrev.mass_spectrum
	====================
	This function takes a list of molecular formula strings and a corresponding numpy array of peak intensities and plots a     mass spectrum using atomic masses calculated via method.  
    
	Use
	----
	mass_spectrum(Y,peak_intensities)
    
    	Returns the figure and axes handles
    
	Parameters
	----------
	Y: A list of molecular formula strings.
	method: the method used to calculate formula mass. See pk.calculate_mass for more information. 
            one of
            'monoisotopic'
            'average'
            'nominal'
            or
            'mz'- plot a list of mz values provided explicitly by the user
    peak_intensities: a list or numpy array of peak intensities 
    normalise: boolean, whether to calculate relative intensity values 
    mz_list: a list or numpy array of mz values, must be provided to use method 'mz'.
    summary_statistics: boolean, if true print the mean, median and standard deviation on the chart.
    **kwargs: key word arguments to plt.plot
    
    """
    ## Set default values for color and linewidth unless they have been given in kwargs
    assert 'c' not in kwargs, 'provide colo(u)r as color'
    if 'color' not in kwargs:
        color = 'black'
    else:
        color = kwargs['color']
    if 'linewidth' not in kwargs:
        linewidth = 0.8
    else:
        linewidth = kwargs['linewidth'] 
    assert len(peak_intensities) > 0, 'Must provide a list or numpy array of peak intensities'
    assert len(peak_intensities) == len(formula_list), 'peak_intensities and formula list must be the same length'
    assert method in ['monoisotopic','average','nominal','mz'], 'Provide a valid method. See docstring for info.'
    if normalise:
        peak_intensities = normalise_intensity(peak_intensities)
    if method == 'mz':
        assert len(mz_list) > 1, 'provide a list of mz values'
        mass = mz_list
    else: 
        mass = calculate_mass(formula_list, method = method)
    # perform sorting of the peak_intensity and mass variables to ensure they are in ascending mass order
    peak_intensities = [x for _,x in sorted(zip(mass,peak_intensities))]
    mass = [y for y,_ in sorted(zip(mass,peak_intensities))]
    plt.plot(mass,peak_intensities,color=color, linewidth=linewidth)
    plt.grid(axis='y', alpha=0.75)
    if method == 'mz':
        plt.xlabel('m/z')
    else:
        plt.xlabel(f"{method[0].upper()}{method[1::]} atomic mass")
    plt.ylabel("Intensity")
    fig = plt.gcf()
    ax = plt.gca()
    return fig,ax



def standardize_formula(qstr,rstr,formula_list):
        
    """ 
	Docstring for function pyKrev.standardize_formula
	====================
	This function takes a list of formula and replaces any instances of qstr with rstr.
	This is an important pre-processing step if isotope names are present in your molecular formula. 
    
	Use
	----
	element_counts(qtr,rstr,Y)
    
	Returns a list of len(Y) in which each item is an atomic formula with qstr replaced by rstr. 
    
	Parameters
	----------
	Y: A list of atomic formula strings. 
    
	qstr: A string with the element name to replace (e.g. '11B')
    
	rstr: A string with the element name to replace with (e.g. 'B')
    
    
    """
    std_formula_list = []
    for formula in formula_list:
        std_formula_list.append(formula.replace(qstr,rstr,1))
    return std_formula_list




from .multi_van_krevelen_plot import multi_van_krevelen_plot
from ..formula import missing_formula
from ..formula import element_counts
from ..formula import element_ratios 

def missing_plot(*groups,**kwargs):
    
    """ 
    
	Docstring for function PyKrev.missing_plot
	====================
	This function takes multiple lists of formula and plots a multi_van_Krevelen with the missing formula
	in each list represented by a different symbol. 
    
	Use
	----
	missing_plot(*Y)
    
	Returns the figure and axes handles and a dictionary containing missing groups.   
    
	Parameters
	----------
	*Y: lists of atomic formula   
    
	**kwargs: key word arguments for multi_van_krevelen. See: pk.multi_van_krevelen_plot()
    
    """
    
    group_labels = []
    if 'group_labels' in kwargs: #if the user has supplied group_labels in kwargs, save them to a variable so unique_groups is labelled correctly 
        group_labels = kwargs['group_labels']
    
    missing_formulas = []
    missing_groups = missing_formula(*groups,group_labels = group_labels)
    for group in missing_groups:
        missing_formulas.append(missing_groups[group])
    
    fig, ax = multi_van_krevelen_plot(missing_formulas,**kwargs) 
    
    return fig, ax, missing_groups
    


def missing_formula(*groups,group_labels = []):
    
    """ 
	Docstring for function pyKrev.missing_formula
    
	====================
	This function compares n lists of molecular formula and outputs a dictionary containing the missing formula in each list.
    
	Use
	----
	missing_formula(list_1,..,list_n)
    
	Returns a dictionary in which each key corresponds to an input list (labelled "group 1...n") 
	and the corresponding value is a list containing the missing formula in that group.  
    
	Parameters
	----------
	*groups: n lists of molecular formula. Each item in the list should be a formula string. 
	 group_labels = list of group label strings of len(groups)
    
    """
    
    default_names = ['group 1','group 2','group 3','group 4','group 5','group 6','group 7','group 8','group 9']

    if not group_labels:
        group_labels = default_names[0:len(groups)]
   
    molecular_formula = dict()
    missing_molecular_formula = dict()
    
    for i,g in zip(group_labels,groups): 
        molecular_formula[i] = g 
    
    #now compare elements of the groups 
    for i in group_labels:
        #create a shallow copy of the molecular formula dictionary 
        temp_molecular_formula = dict.copy(molecular_formula)
        #create a set out of the unique formulas in the current group 
        current_group = set(molecular_formula[i])
        #remove the current group from the shallow copy
        del temp_molecular_formula[i]
        #create a list out of the formulas in the shallow copy
        other_groups = list(temp_molecular_formula.values()) 
        #find the formula that are shared by all other_groups
        common_groups = set(other_groups[0])
        for og in other_groups: 
            common_groups = common_groups & set(og)  # the & operator returns intersecting values of sets
        #find the common formula that aren't present in the current_group 
        missing_formula  = []
        for formula in common_groups:
            if formula in current_group:
                pass
            else:
                missing_formula.append(formula)
        #add it to the unique_molecular_formula dict 
        missing_molecular_formula[i] = missing_formula
    
    
    return missing_molecular_formula
    


from .calculate_mass import calculate_mass
from .element_counts import element_counts
import numpy as np
def kendrick_mass_defect(formula_list,mz_list,base = ['CH2'], rounding = 'even'):
    """ 
	Docstring for function pyKrev.kendrick_mass_defect
	====================
	This function takes a list of molecular formula and their corresponding mz_values and calculates kendrick mass defect scores.  
    
	Use
	----
	kendrick_mass_defect(Y,X)
    
	Returns a tuple contains two numpy arrays of len(Y). The first contains the kendrickMass and the second contains the kendrickMassDefect.
    
	Parameters
	----------
	Y: A list of molecular formula. 
    X: A list of mz_values of len(Y). 
    
    base: Atom group used to define the Kendrick mass.
    rounding: The method of rounding to use when calculating kendrick mass defect. One of 
        - even: for values exactly halfway between rounded decimal values, numpy rounds to the nearest even value. 
                Thus 1.5 and 2.5 round to 2.0, -0.5 and 0.5 round to 0.0, etc.
        - integer: for values exactly halfway between rounded decimal values, numpy rounds up to the nearest integer value. 
                Thus 1.5 becomes 2.0, 2.5 becomes 3.0 
    Info
	----------
    Calculation taken from Hughey et al (2001) 
    "Kendrick Mass Defect Spectrum: A Compact Visual Analysis for Ultrahigh-Resolution Broadband Mass Spectra"
    Note: Rounding calculations may lead to "edge effects" in complex datasets with many peaks. 
    We recommend experimenting with both even and integer rounding methods when making kmd plots.

    """
    
    #Formula and mz lists must be the same length
    assert(len(formula_list)==len(mz_list))
    if type(mz_list) == list:
        mz_list = np.array(mz_list)
    if type(base) != list:
        raise AssertionError("Base must be provided as a string within a list. E.g. ['CH2']")
    #Check for valid rounding method 
    assert(rounding in ['even','integer']), "provide a valid rounding method"

    kendrickMass = np.empty(len(formula_list))
    kendrickMassDefect = np.empty(len(formula_list))

    nominalBase = calculate_mass(base, method = 'nominal')
    exactBase = calculate_mass(base, method = 'monoisotopic')
    kendrickMass = np.array(mz_list) * (nominalBase/exactBase)
    if rounding == 'even':
        kendrickMassDefect  = np.around(kendrickMass) - kendrickMass
    elif rounding == 'integer':
        kendrickMassDefect  = np.rint(kendrickMass) - kendrickMass
    
    return kendrickMass, kendrickMassDefect


import pandas as pd
import numpy as np 
from ..diversity import ordination_matrix
def read_batch_formularity(report_name, ordination =True, impute_value = 'nan'):
    """ 
	Docstring for function PyKrev.read_batch_formularity
	====================
	This function reads in data from the report csv file produced by formularity software when performing batch assignment with alignment.
	The function filters out any formula that do not have C and H atoms.
    
	Use
	----
	read_batch_formularity(report_name)
    
	Returns a dataframe with the formula strings, masses, compound classes and mass errors of the overall alignment and their corresponding peak 
    intensities in each spectra, and an ordination matrix of type pd.Dataframe with formula strings as column headers and spectra as rows (see 
    pk.ordination_matrix) (optional) 
    
	Parameters
	----------
	report_name: name of csv file that the formularity report to be read is saved as. 
	impute_value: value to impute for 0 peak intensities in the output ordination matrix (default 0), see pk.Ordination_matrix)
    ordination: boolean, compute an ordination matrix using pk.ordination_matrix?
    """
    #read in csv to pandas
    report = pd.read_csv(report_name)
    #sum C and C13 to get total C number
    C = report['C'] + report['C13']
    H = report['H']
    O = report['O']
    N = report['N'] 
    S = report['S']
    P = report['P']
    #dimensions of report (number of formula, samples)
    x,y = report.shape
    #list to store all formula in
    molecular_formula = []
    #turn atom numbers into formula strings 
    for n in range(0,len(C)):
        if C[n] == 0 and H[n] == 0: #If there isn't a count for C and H don't include the formula
            temp_formula = np.nan
        else:
            temp_formula = 'C'+str(C[n])+'H'+str(H[n]) #add C and H then ... 
            if N[n] > 0:
                temp_formula = temp_formula + 'N' + str(N[n])
            if O[n] > 0:
                temp_formula = temp_formula + 'O' + str(O[n])
            if P[n] > 0:
                temp_formula = temp_formula + 'P' + str(P[n])
            if S[n] > 0:
                temp_formula = temp_formula + 'S' + str(S[n])
            
        molecular_formula.append(temp_formula)
    
    #convert to numpy array so we can use boolean indexing
    molecular_formula = np.array(molecular_formula)
    #remove nan values
    filter_formula = molecular_formula[molecular_formula != 'nan']
    #add to the original dataframe to preserve indexing
    report['all_formula'] = molecular_formula
    #make new dataframe
    formulaData = pd.DataFrame(index=None)
    #copy across formula, mass, cclass and mass error data with boolean indexing
    formulaData['formula'] = report['all_formula'][molecular_formula != 'nan']
    formulaData['mass'] = report['Mass'][molecular_formula != 'nan']
    formulaData['cclass'] = report['Class'][molecular_formula != 'nan']
    formulaData['massError'] = report['Error_ppm'][molecular_formula != 'nan']
    
    spectraNames = []
    spectraFormula = []
    spectraIntensities = []
    #column 15 is the first sample ID column
    for i in range(14,y):
        #save the peak intensities after boolean indexing has been applied
        intensity = report.iloc[:,i][molecular_formula != 'nan']
        formulaData[report.columns[i]] = intensity
        spectraNames.append(report.columns[i])
        spectraIntensities.append(np.array(intensity[intensity > 0]))
        spectraFormula.append(list(filter_formula[intensity > 0]))
    
    ordinationMat = []
    if ordination == True:
        ordinationMat = ordination_matrix(molecular_formulas = spectraFormula,peak_intensities = spectraIntensities, group_names = spectraNames, impute_value = impute_value)
        #This will happen if there are identical formula in the molecular formula list
        if ordinationMat.shape[1] != formulaData.shape[0]:
            print('Warning, duplicate formula assignments detected. Ordination matrix will report values for first formula it encounters.')
    return formulaData, ordinationMat



import numpy as np
def filter_spectral_interference(mass_list,formula_list,peak_intensities,tol = 2,verbose = True):
    """ 
	Docstring for function pykrev.filter_spectral_interference
	====================
	This function takes a list or numpy array of masses and a corresponding list of formula strings 
    and a corresponding list or numpy array of peak intensities
    and returns filtered copies of all arrays with suspected 
    spectral interferences due to high molecular weight doubly charged ions, removed.
    
	Use
	----
	filter_spectral_interference(Y,X,Z)
    
	Returns a tuple containing an array of len(Y[filter]) in which each item is the filtered mass, and a list of len(Y[filter]) in which each item 
    is the filtered formula and an array of len(Y[filter]) in which each item is a filtered peak_intensity. 
    
	Parameters
	----------
	Y: A list or numpy array of masses.
    X: A list of molecular formula strings.
    Z: A list or numpy array of peak intensities.
    tol: Integer of float,the tolerance (in ppm) used to identify monoisotopic 
    verbose: Booloean, print text to the console.
    
    Note: for more information on this algorithm please refer to Patriarca, Claudia, and Jeffrey A. Hawkes. 
    "High Molecular Weight Spectral Interferences in Mass Spectra of Dissolved Organic Matter." 
    Journal of the American Society for Mass Spectrometry (2020).
    """    
    assert len(mass_list) == len(formula_list) == len(peak_intensities), "All input arrays must be of equal size"
    mass_list = np.array(mass_list)
    formula_list = np.array(formula_list)
    peak_intensities = np.array(peak_intensities)
    mass_defect = mass_list - np.floor(mass_list)
    c13d2 = (13.00335-12)/2
    flags = np.zeros(len(mass_list))
    for i in range(0,len(mass_list)):
        if mass_defect[i] > 0.4 and mass_defect[i] < 0.8:
            flags[i] = 1
            mono_mass = mass_list[i] - c13d2
            mass_diff = abs(mass_list - mono_mass)
            mono_pos = mass_diff == min(mass_diff)
            if min(mass_diff)/mass_list[i] * 1e6 < tol:
                if peak_intensities[mono_pos]/peak_intensities[i] < 10:
                    flags[mono_pos] = 2
    spectralFilter = flags == 0 
    filter_mass = mass_list[spectralFilter]
    filter_formula = formula_list[spectralFilter]
    filter_peak_intensities = peak_intensities[spectralFilter]
    
    if verbose == True:
        print(f"{len(mass_list)-len(filter_mass)} masses removed.")
    
    return filter_mass, filter_formula, filter_peak_intensities


def element_counts(formula_list):
    
    """ 
	Docstring for function pyKrev.element_counts
	====================
	This function takes a list of formula strings and gives atomic counts for C,H,N,O,P & S.
    
	Use
	----
	element_counts(Y)
    
	Returns a list of len(Y) in which each element is a dictionary containing the atomic counts. 
    
	Parameters
	----------
    
	Y: A list of elemental formula strings. All integers must be standard script (e.g. CH2O5). 
    
    """
    
    elements=['C','H','N','O','P','S']
    count_list = []
    for formula in formula_list:
        element_numbers = dict()
        for element in elements:
            element_numbers[element] = 0
            alpha_idx = formula.find(element) #find this first instance of the element name. If the formula is given with alphabetically similar two digit element names e.g. Co before single digit names e.g. C. 
                                           #it won't work. In theory this shouldn't happen because formulas should be listed alphabetically, therefore single character names will come before two character names. 
            if alpha_idx > -1: # if the element was found in the formula   
                element_numbers[element] = 1 #it must have at least 1 atom 
                for i in range(alpha_idx+2,(len(formula)+1)):
                    try:
                        element_numbers[element] = int(formula[alpha_idx+1:i]) #Count the number of atoms belonging to this element starting with the next character, and progressively making the character window bigger
                    except ValueError: # this occurs when you have reached the next element name
                        break
        count_list.append(element_numbers)
    return count_list



from .multi_van_krevelen_plot import multi_van_krevelen_plot
from ..formula import unique_formula
from ..formula import element_counts
from ..formula import element_ratios 

def unique_plot(*groups, **kwargs):
    
    """ 
	Docstring for function PyKrev.unique_plot
	====================
	This function takes multiple lists of formula and plots a multi_van_Krevelen with the unique formula
	in each list represented by a different symbol. 
    
	Use
	----
	unique_plot(Y*)
    
	Returns figure and ax handles and the unique_groups produced by pk.unique_formula().   
    
	Parameters
	----------
	Y*: lists of atomic formula   
    
	**kwargs: key word arguments for pk.multi_van_krevelen_plot. See. pk.multi_van_krevelen_plot for more information.
    
    """ 
    #assert ('group_labels' in kwargs), 'you must provide group_labels'
    group_labels = []
    if 'group_labels' in kwargs: #if the user has supplied group_labels in kwargs, save them to a variable so unique_groups is labelled correctly 
        group_labels = kwargs['group_labels']
    
    unique_formulas = []
    unique_groups = unique_formula(*groups,group_labels = group_labels)
    
    for group in unique_groups:
       unique_formulas.append(unique_groups[group])
    
    fig,ax = multi_van_krevelen_plot(unique_formulas, **kwargs)
    
    return fig,ax,unique_groups
    


from .element_counts import element_counts
import numpy as np
def aromaticity_index(formula_list, index_type = 'rAI'):
    
          
    """ 
	Docstring for function pyKrev.aromaticity_index
	====================
	This function takes a list of molecular formula strings and returns the aromaticity index.
    
	Use
	----
	aromaticity_index(Y)
    
	Returns a numpy array of len(Y) in which each item is the aromaticity index.  
    
	Parameters
	----------
	Y: A list of molecular formula strings. 
    
	index_type: One of the following strings:
        -'rAI' - reformulated aromaticity index see Mendelez-Perez et al. (2016)
        -'rAImod'- reformulated aromaticity index with modified oxygen coefficient 
        -'AI' - aromaticity index see Koch and Dittmar (2006) 
        -'AImod'- aromaticity index with modified oxygen coefficient see Koch and Dittmar (2006)
    
	Info
	----------
	Reformulated aromaticity index see Mendelez-Perez et al. (2016),
	"A reformulated aromaticity index equation under consideration for non-aromatic 
	and non-condensed aromatic cyclic carbonyl compounds."
    Aromaticity index see Koch and Dittmar (2006)
    "From mass to structure: an aromaticity index for
     high-resolution mass data of natural organic matter"

    """    
    
    count_list = element_counts(formula_list)
    assert index_type in ['rAI','rAImod','AI','AImod'], 'supply a valid index type, read doc string for more info'
    AI_array = np.array([])
    warning = 0 
    
    for count in count_list: 
        try:
            if index_type == 'rAImod':
                AI_counts = (1 + 1/2*((count['C']*2)-count['H']-(count['O']*1)-(count['S']*2)-count['N']-count['P']))/count['C']
            elif index_type == 'rAI':
                AI_counts = (1 + 1/2*((count['C']*2)-count['H']-(count['O']*2)-(count['S']*2)-count['N']-count['P']))/count['C']
            elif index_type == 'AImod':
                AI_counts = (1 + count['C'] - (count['H']*0.5)-(count['O']*0.5)- count['S'])/(count['C'] - (count['O']*0.5) - count['N'] - count['S'] - count['P'])
            elif index_type == 'AI':
                AI_counts = (1 + count['C'] -count['O']- count['S'] - (0.5*count['H']))/(count['C'] - count['O'] - count['N'] - count['S'] - count['P'])
        except ZeroDivisionError:
            AI_counts = np.nan
            print(f"Warning Zero Division Encountered, NaN value returned. {count}")
        if AI_counts < 0: 
            warning = 1 
            AI_counts = 0
            
        AI_array = np.append(AI_array,AI_counts)
        
    if warning == 1:
        print('Warning: negative ai counts detected and set to zero.')
        
    return AI_array


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pykrev", 
    version="1.1.1",
    author="Ezra Kitson",
    author_email="ezrakitson@ed.ac.uk",
    description="FT-MS data analysis in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kzra/pykrev",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    include_package_data=True,
)


import numpy as np
from .element_counts import element_counts
def calculate_mass(formula_list, method = 'monoisotopic'):     
    """ 
	Docstring for function pyKrev.calculate_mass
	====================
	This function takes a list of molecular formula strings and calculates the nominal mass for each formula.
    
	Use
	----
	approximate mass(Y)
    
	Returns a numpy array of len(Y) in which each item is the calculated mass 
    
	Parameters
	----------
	Y: A list of molecular formula strings. 
    
    Method: The type of mass calculation to perform. One of:
        - monoisotopic, i.e. based on the exact mass of the most abundant isotope. 
        - nominal, i.e. the mass of the most abundant isotope rounded to the nearest integer.
        - average, i.e. mass taking into account weighted abundance of all natural isotopes of the element. 
    
    Note:  exact masses taken from: https://www.intechopen.com/books/mass-spectrometry/interpretation-of-mass-spectra
           average masses taken from: http://physics.nist.gov/PhysRefData/Compositions/
    
    """
    count_list = element_counts(formula_list)
    
    if method == 'monoisotopic':
        element_masses = {'C': 12.00000, 'H': 1.007825, 'O':15.994915,'N': 14.003074,'S': 31.972072, 'P': 30.97376}
    elif method == 'nominal':
        element_masses = {'C': 12, 'H': 1, 'O':16,'N': 14,'S': 32, 'P': 31}
    elif method == 'average':
        element_masses = {'C': 12.010736, 'H': 1.007941, 'O':15.999405,'N': 14.006743,'S': 32.066085, 'P': 30.973762}
    else:
        raise Exception("Method not recognised.")
        
    mass_list = np.empty(len(count_list))
    for i in range(0,len(count_list)):
        mass_list[i] = count_list[i]['C'] * element_masses['C'] + count_list[i]['H'] * element_masses['H'] + count_list[i]['N'] * element_masses['N'] + count_list[i]['O'] * element_masses['O'] + count_list[i]['P'] * element_masses['P'] + count_list[i]['S'] * element_masses['S']
        
    return mass_list